//
// Test app by George Harth <george@modeblue.com>
//

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace ESignalStressTest
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private IESignal.HooksClass esignal;

		private string[] symbols =
		{
			"GE","AWE","PFE","TWX","MOT","EMC","C","TXN","HPQ","XOM","MU","F",
			"DIS","AMD","SBC","WMT","JPM","TYC","BMY","JNJ","HD","MRK","VZ","VIA.B","NEM",
			"SGP","BAC","WYE","GM","USB","MO","AA","T","KO","ET","AIG","AV","IBM","MCD",
			"CD","MDT","FON","ABT","AXP","XRX","GPS","BLS","MER","MWD","PEP","LUV","BSX",
			"FDC","NSM","KRB","HAL","TGT","CMX","WFC","DD","KSS","ONE","BBY","LOW","EDS",
			"KR","CFC","WM","DUK","SLB","LLY","RIG","FNM","WB","TER","JCP","GS","A","CC",
			"MAT","LMT","FCX","DOW","GDT","ADI","BK","CCU","LTD","PG","EK","BAX","WAG","HON",
			"X","PVN","ADM","SWY","BA","MMM","CA",
		};
		private symbolInfo[] symbolInfoArray = null;
			
		private System.Windows.Forms.TextBox usernameTextBox;
		private System.Windows.Forms.Button startBtn;
		private System.Windows.Forms.ListView listView1;
		private System.Windows.Forms.Label lastTimeLabel;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			esignal = new IESignal.HooksClass();
			esignal.OnBarsReceived += new IESignal._IHooksEvents_OnBarsReceivedEventHandler(esignal_OnBarsReceived);
			esignal.OnBarsChanged += new IESignal._IHooksEvents_OnBarsChangedEventHandler(esignal_OnBarsChanged);

			listView1.View = View.Details;
			listView1.Columns.Add("", 25, HorizontalAlignment.Left);
			listView1.Columns.Add("Symbol", 50, HorizontalAlignment.Left);
			listView1.Columns.Add("Open", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("High", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Low", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Close", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Volume", 60, HorizontalAlignment.Right);
			listView1.Columns.Add("Time", 60, HorizontalAlignment.Center);

			symbolInfoArray = new symbolInfo[symbols.Length];
			for (int idx = 0; idx < symbols.Length; ++idx) 
			{
				symbolInfoArray[idx] = new symbolInfo();
				symbolInfoArray[idx].symbol = symbols[idx];

				ListViewItem item = new ListViewItem((idx + 1).ToString());
				item.SubItems.Add(symbols[idx]);
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				item.SubItems.Add("");
				listView1.Items.Add(item);
				symbolInfoArray[idx].item = item;	// Store it for faster access later
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			esignal.ReleaseAllHistory();
			esignal.ReleaseAllTimeSales();

			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.startBtn = new System.Windows.Forms.Button();
			this.listView1 = new System.Windows.Forms.ListView();
			this.usernameTextBox = new System.Windows.Forms.TextBox();
			this.lastTimeLabel = new System.Windows.Forms.Label();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// startBtn
			// 
			this.startBtn.Location = new System.Drawing.Point(400, 8);
			this.startBtn.Name = "startBtn";
			this.startBtn.Size = new System.Drawing.Size(64, 24);
			this.startBtn.TabIndex = 1;
			this.startBtn.Text = "Start";
			this.startBtn.Click += new System.EventHandler(this.startBtn_Click);
			// 
			// listView1
			// 
			this.listView1.CausesValidation = false;
			this.listView1.FullRowSelect = true;
			this.listView1.GridLines = true;
			this.listView1.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.Nonclickable;
			this.listView1.Location = new System.Drawing.Point(8, 52);
			this.listView1.Name = "listView1";
			this.listView1.Size = new System.Drawing.Size(456, 310);
			this.listView1.TabIndex = 2;
			// 
			// usernameTextBox
			// 
			this.usernameTextBox.Location = new System.Drawing.Point(316, 8);
			this.usernameTextBox.Name = "usernameTextBox";
			this.usernameTextBox.Size = new System.Drawing.Size(80, 20);
			this.usernameTextBox.TabIndex = 0;
			this.usernameTextBox.Text = "";
			// 
			// lastTimeLabel
			// 
			this.lastTimeLabel.Location = new System.Drawing.Point(16, 10);
			this.lastTimeLabel.Name = "lastTimeLabel";
			this.lastTimeLabel.Size = new System.Drawing.Size(192, 18);
			this.lastTimeLabel.TabIndex = 3;
			this.lastTimeLabel.Text = "Last Time =";
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(266, 10);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(48, 16);
			this.label1.TabIndex = 4;
			this.label1.Text = "usrname";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 34);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(444, 16);
			this.label2.TabIndex = 5;
			this.label2.Text = "Note:  The OHLCV information in the grid is for the most recent 1 minute bar";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(472, 371);
			this.Controls.Add(this.usernameTextBox);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.lastTimeLabel);
			this.Controls.Add(this.listView1);
			this.Controls.Add(this.startBtn);
			this.Name = "Form1";
			this.Text = "Multiple Stock Streams Test";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		private void startBtn_Click(object sender, System.EventArgs e)
		{
			esignal.SetApplication(usernameTextBox.Text);

			if (esignal.IsEntitled == 0)
			{
				MessageBox.Show("Not entitled");
			}
			else 
			{
				esignal.ReleaseAllHistory();
				esignal.ReleaseAllTimeSales();
				foreach (symbolInfo info in symbolInfoArray)
				{
					// We are going to display the OHLC of 1 minute bars, so ask for them
					info.historyHandle = esignal.get_RequestHistory(info.symbol, "1", 
						IESignal.barType.btBARS, 20, 0, 0);
				}
				startBtn.Enabled = false;
				usernameTextBox.Enabled = false;
			}
		}

		private int historyHandleToIndex(int lHandle)
		{
			for (int idx = 0; idx < symbolInfoArray.Length; ++idx)
			{
				if (lHandle == symbolInfoArray[idx].historyHandle)
				{
					return idx;
				}
			}
			return -1;
		}

		private void esignal_OnBarsReceived(int lHandle)
		{
			int index = historyHandleToIndex(lHandle);
			if ((index >= 0) && (esignal.get_GetNumBars(lHandle) > 0))
			{
				IESignal.BarData bd = esignal.get_GetBar(lHandle, 0);
				ListViewItem item = symbolInfoArray[index].item;
				item.SubItems[2].Text = bd.dOpen.ToString();
				item.SubItems[3].Text = bd.dHigh.ToString();
				item.SubItems[4].Text = bd.dLow.ToString();
				item.SubItems[5].Text = bd.dClose.ToString();
				item.SubItems[6].Text = bd.dVolume.ToString();
				item.SubItems[7].Text = bd.dtTime.ToShortTimeString();
				lastTimeLabel.Text = "Last Time = " + bd.dtTime.ToShortTimeString() + 
					" [" + symbolInfoArray[index].symbol + "]";
			}
		}

		private void esignal_OnBarsChanged(int lHandle)
		{
			int index = historyHandleToIndex(lHandle);
			if ((index >= 0) && (esignal.get_GetNumBars(lHandle) > 0))
			{
				IESignal.BarData bd = esignal.get_GetBar(lHandle, 0);
				ListViewItem item = symbolInfoArray[index].item;
				item.SubItems[2].Text = bd.dOpen.ToString();
				item.SubItems[3].Text = bd.dHigh.ToString();
				item.SubItems[4].Text = bd.dLow.ToString();
				item.SubItems[5].Text = bd.dClose.ToString();
				item.SubItems[6].Text = bd.dVolume.ToString();
				item.SubItems[7].Text = bd.dtTime.ToShortTimeString();
				lastTimeLabel.Text = "Last Time = " + bd.dtTime.ToShortTimeString() + 
					" [" + symbolInfoArray[index].symbol + "]";
			}
		}
	}

	public class symbolInfo
	{
		public int historyHandle = -1;
		public string symbol = "";
		public ListViewItem item = null;
	}
}
